//+------------------------------------------------------------------+
//|                                                          GNG.mq5 |
//|                                             Copyright 2010, alsu |
//|                                                 alsufx@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, alsu"
#property link      "alsufx@gmail.com"
#property version   "1.00"
#property script_show_inputs

#include <GNG/GNG.mqh>

//--- number of input samples used for training
input int      samples=1000;

//--- algorithm parameters
input int lambda=20;
input int age_max=15;
input double alpha=0.5;
input double beta=0.0005;
input double eps_w=0.05;
input double eps_n=0.0006;
input int max_nodes=100;

//---global variables
CGNGAlgorithm *GNGAlgorithm;
int rsi_handle;
int input_dimension;
int _samples;
double RSI_buffer[];
datetime time[];
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   int i,j;
   int window=ChartWindowFind(0,"GNG_dummy");

//--- adjust number of samples to be sure that CopyBuffer() will return valid data  
   _samples=samples+input_dimension+10;
   if(_samples>Bars(_Symbol,_Period)) _samples=Bars(_Symbol,_Period);

//--- this will be our input data
   rsi_handle=iRSI(NULL,0,8,PRICE_CLOSE);
   CopyBuffer(rsi_handle,0,1,_samples,RSI_buffer);

//---insure we have enough data for input
   _samples=_samples-input_dimension-10;

//--- get open times for first 100 bars where we shall draw objects
   CopyTime(_Symbol,_Period,0,100,time);

//--- create algorithm object and set the dimension of input data
   GNGAlgorithm=new CGNGAlgorithm;
   input_dimension=2;

//--- data vectors
   double v[],v1[],v2[];
   ArrayResize(v,input_dimension);
   ArrayResize(v1,input_dimension);
   ArrayResize(v2,input_dimension);

   for(i=0;i<input_dimension;i++)
     {
      v1[i] = RSI_buffer[i];
      v2[i] = RSI_buffer[i+3];
     }

//--- initialize algorithm
   GNGAlgorithm.Init(input_dimension,v1,v2,lambda,age_max,alpha,beta,eps_w,eps_n,max_nodes);

//-- draw rectangle and info labels in the upper right corner of the graph
   ObjectCreate(0,"GNG_rect",OBJ_RECTANGLE,window,time[0],0,time[99],100);
   ObjectSetInteger(0,"GNG_rect",OBJPROP_BACK,true);
   ObjectSetInteger(0,"GNG_rect",OBJPROP_COLOR,DarkGray);
   ObjectSetInteger(0,"GNG_rect",OBJPROP_BGCOLOR,DarkGray);

   ObjectCreate(0,"Label_samples",OBJ_LABEL,window,0,0);
   ObjectSetInteger(0,"Label_samples",OBJPROP_ANCHOR,ANCHOR_RIGHT_UPPER);
   ObjectSetInteger(0,"Label_samples",OBJPROP_CORNER,CORNER_RIGHT_UPPER);
   ObjectSetInteger(0,"Label_samples",OBJPROP_XDISTANCE,10);
   ObjectSetInteger(0,"Label_samples",OBJPROP_YDISTANCE,10);
   ObjectSetInteger(0,"Label_samples",OBJPROP_COLOR,Red);
   ObjectSetString(0,"Label_samples",OBJPROP_TEXT,"Total samples: 2");

   ObjectCreate(0,"Label_neurons",OBJ_LABEL,window,0,0);
   ObjectSetInteger(0,"Label_neurons",OBJPROP_ANCHOR,ANCHOR_RIGHT_UPPER);
   ObjectSetInteger(0,"Label_neurons",OBJPROP_CORNER,CORNER_RIGHT_UPPER);
   ObjectSetInteger(0,"Label_neurons",OBJPROP_XDISTANCE,10);
   ObjectSetInteger(0,"Label_neurons",OBJPROP_YDISTANCE,25);
   ObjectSetInteger(0,"Label_neurons",OBJPROP_COLOR,Red);
   ObjectSetString(0,"Label_neurons",OBJPROP_TEXT,"Total neurons: 2");

//--- main cycle for GNG training, starting with i=2 since 2 samples have
//--- already been used
   for(i=2;i<_samples;i++)
     {
      //--- fill in sample vector (gap of 3 bars needed for lower correlation
      //--- between vector components - for better visualization)
      for(j=0;j<input_dimension;j++)
         v[j]=RSI_buffer[i+j*3];

      //--- show sample on chart
      ObjectCreate(0,"Sample_"+IntegerToString(i),OBJ_ARROW,window,time[int(v[0])],v[1]);
      ObjectSetInteger(0,"Sample_"+IntegerToString(i),OBJPROP_ARROWCODE,158);
      ObjectSetInteger(0,"Sample_"+IntegerToString(i),OBJPROP_COLOR,Blue);
      ObjectSetInteger(0,"Sample_"+IntegerToString(i),OBJPROP_BACK,true);

      //--- update label
      ObjectSetString(0,"Label_samples",OBJPROP_TEXT,"Total samples: "+string(i+1));

      //--- pass sample vector to the algorithm object
      GNGAlgorithm.ProcessVector(v);

      //--- delete old neurons and connections points from graph
      for(j=ObjectsTotal(0)-1;j>=0;j--)
        {
         string name=ObjectName(0,j);
         if(StringFind(name,"Neuron_")>=0)
           {
            ObjectDelete(0,name);
           }
         else if(StringFind(name,"Connection_")>=0)
           {
            ObjectDelete(0,name);
           }
        }

      double weights[];
      CGNGNeuron *tmp,*W1,*W2;
      CGNGConnection *tmpc;

      GNGAlgorithm.Neurons.FindWinners(W1,W2);

      //--- draw neurons
      tmp=GNGAlgorithm.Neurons.GetFirstNode();
      while(CheckPointer(tmp))
        {
         tmp.Weights(weights);

         ObjectCreate(0,"Neuron_"+IntegerToString(tmp.uid),OBJ_ARROW,window,time[int(weights[0])],weights[1]);
         ObjectSetInteger(0,"Neuron_"+IntegerToString(tmp.uid),OBJPROP_ARROWCODE,159);

         //--- winner Lime, second winner Green and other neurons Red
         if(tmp==W1) ObjectSetInteger(0,"Neuron_"+IntegerToString(tmp.uid),OBJPROP_COLOR,Lime);
         else if(tmp==W2) ObjectSetInteger(0,"Neuron_"+IntegerToString(tmp.uid),OBJPROP_COLOR,Green);
         else ObjectSetInteger(0,"Neuron_"+IntegerToString(tmp.uid),OBJPROP_COLOR,Red);

         ObjectSetInteger(0,"Neuron_"+IntegerToString(tmp.uid),OBJPROP_BACK,false);

         tmp=GNGAlgorithm.Neurons.GetNextNode();
        }
      ObjectSetString(0,"Label_neurons",OBJPROP_TEXT,"Total neurons: "+string(GNGAlgorithm.Neurons.Total()));

      //--- draw connections
      tmpc=GNGAlgorithm.Connections.GetFirstNode();
      while(CheckPointer(tmpc))
        {
         int x1,x2,y1,y2;

         tmp=GNGAlgorithm.Neurons.Find(tmpc.uid1);
         tmp.Weights(weights);
         x1=int(weights[0]);
         y1=int(weights[1]);

         tmp=GNGAlgorithm.Neurons.Find(tmpc.uid2);
         tmp.Weights(weights);
         x2=int(weights[0]);
         y2=int(weights[1]);

         ObjectCreate(0,"Connection_"+IntegerToString(tmpc.uid1)+"_"+IntegerToString(tmpc.uid2),OBJ_TREND,window,time[x1],y1,time[x2],y2);
         ObjectSetInteger(0,"Connection_"+IntegerToString(tmpc.uid1)+"_"+IntegerToString(tmpc.uid2),OBJPROP_WIDTH,1);
         ObjectSetInteger(0,"Connection_"+IntegerToString(tmpc.uid1)+"_"+IntegerToString(tmpc.uid2),OBJPROP_STYLE,STYLE_DOT);
         ObjectSetInteger(0,"Connection_"+IntegerToString(tmpc.uid1)+"_"+IntegerToString(tmpc.uid2),OBJPROP_COLOR,Yellow);
         ObjectSetInteger(0,"Connection_"+IntegerToString(tmpc.uid1)+"_"+IntegerToString(tmpc.uid2),OBJPROP_BACK,false);

         tmpc=GNGAlgorithm.Connections.GetNextNode();
        }

      ChartRedraw();
     }
  }
//+------------------------------------------------------------------+
